searchNodes=[{"ref":"EctoDiff.html","title":"EctoDiff","module":"EctoDiff","type":"module","doc":"Generates a data structure describing the difference between two ecto structs. For details on how to generate an EctoDiff.t/0 struct, see: diff/2. For details on what the generated struct looks like, see: EctoDiff.t/0."},{"ref":"EctoDiff.html#diff/2","title":"EctoDiff.diff/2","module":"EctoDiff","type":"function","doc":"Returns an EctoDiff.t/0 describing the difference between two given ecto structs. The &quot;previous&quot; struct can be nil, to represent an insert operation. Examples A new struct being inserted: iex&gt; {:ok, pet} = %{name: &quot;Spot&quot;} |&gt; Pet.new() |&gt; Repo.insert() iex&gt; {:ok, diff} = EctoDiff.diff(nil, pet) iex&gt; diff #EctoDiff&lt; struct: Pet, primary_key: %{id: 1}, effect: :added, previous: #Pet&lt;&gt;, current: #Pet&lt;&gt;, changes: %{ id: {nil, 1}, name: {nil, &quot;Spot&quot;} } &gt; A struct being updated: iex&gt; {:ok, pet} = %{name: &quot;Spot&quot;} |&gt; Pet.new() |&gt; Repo.insert() iex&gt; {:ok, updated_pet} = pet |&gt; Pet.update(%{name: &quot;McFluffFace&quot;}) |&gt; Repo.update() iex&gt; {:ok, diff} = EctoDiff.diff(pet, updated_pet) iex&gt; diff #EctoDiff&lt; struct: Pet, primary_key: %{id: 1}, effect: :changed, previous: #Pet&lt;&gt;, current: #Pet&lt;&gt;, changes: %{ name: {&quot;Spot&quot;, &quot;McFluffFace&quot;} } &gt; A nested has_many association being updated: iex&gt; {:ok, initial_pet} = ...&gt; %{name: &quot;Spot&quot;, skills: [%{name: &quot;Eating&quot;}, %{name: &quot;Sleeping&quot;}, %{name: &quot;Scratching&quot;}]} ...&gt; |&gt; Pet.new() ...&gt; |&gt; Repo.insert() iex&gt; [eating_id, sleeping_id, _scratching_id] = Enum.map(initial_pet.skills, &amp; &amp;1.id) iex&gt; {:ok, updated_pet} = ...&gt; initial_pet ...&gt; |&gt; Pet.update(%{skills: [%{id: eating_id}, %{id: sleeping_id, level: 2}, %{name: &quot;Meowing&quot;}]}) ...&gt; |&gt; Repo.update() iex&gt; {:ok, diff} = EctoDiff.diff(initial_pet, updated_pet) iex&gt; diff #EctoDiff&lt; struct: Pet, primary_key: %{id: 1}, effect: :changed, previous: #Pet&lt;&gt;, current: #Pet&lt;&gt;, changes: %{ skills: [ #EctoDiff&lt; struct: Skill, primary_key: %{id: 2}, effect: :changed, previous: #Skill&lt;&gt;, current: #Skill&lt;&gt;, changes: %{ level: {1, 2} } &gt;, #EctoDiff&lt; struct: Skill, primary_key: %{id: 3}, effect: :deleted, previous: #Skill&lt;&gt;, current: #Skill&lt;&gt;, changes: %{} &gt;, #EctoDiff&lt; struct: Skill, primary_key: %{id: 4}, effect: :added, previous: #Skill&lt;&gt;, current: #Skill&lt;&gt;, changes: %{ id: {nil, 4}, pet_id: {nil, 1}, name: {nil, &quot;Meowing&quot;} } &gt; ] } &gt;"},{"ref":"EctoDiff.html#t:effect/0","title":"EctoDiff.effect/0","module":"EctoDiff","type":"type","doc":"The type of change for a given struct. Each EctoDiff.t/0 struct will have a field describing what happened to the given ecto struct. The values can be one of the following: :added - This struct is new and was not present previously. This happens when the primary struct, or an associated struct, was added during an update or insert. :deleted - This struct was previously present, but no longer is. :changed - This struct existed previously and still does, but some of its fields and/or associations have changed. :replaced - This struct was replaced with a completely new one. This happens with belongs_to or embeds_one associations with the on_replace: :nilify option set."},{"ref":"EctoDiff.html#t:t/0","title":"EctoDiff.t/0","module":"EctoDiff","type":"type","doc":"Describes all changes made during an insert or update operation. The following fields should be considered public: struct - The module atom of the ecto schema being diffed. primary_key - The primary key(s) of the ecto struct. This is a map of all primary keys in case of composite keys. For most common use-cases this will just be the map %{id: id}. changed - A map representing all changes made. The keys will be fields and associations defined in the ecto schema, but only fields and associations with changes will be present. For changed fields, the value will be a tuple representing the previous and new values (i.e. {previous, new}). For associations, the value will be another EctoDiff.t/0 struct for cardinality &quot;one&quot; associations, or a list of EctoDiff.t/0 structs for cardinality &quot;many&quot; associations. effect - The type of change for this ecto struct. See effect/0 for details. previous - The previous struct itself. current - The current (new) struct itself."},{"ref":"readme.html","title":"EctoDiff","module":"readme","type":"extras","doc":"EctoDiff Generates a data structure that describes the differences between two ecto structs. The primary use-case is to track what changed after calling Repo.insert or Repo.update, especially in conjunction with complex or deeply nested cast_assoc associations."},{"ref":"readme.html#installation","title":"EctoDiff - Installation","module":"readme","type":"extras","doc":"The package can be installed by adding ecto_diff to your list of dependencies in mix.exs: def deps do [ {:ecto_diff, &quot;~&gt; 0.2.0&quot;} ] end"},{"ref":"readme.html#basic-usage","title":"EctoDiff - Basic Usage","module":"readme","type":"extras","doc":"To demonstrate the basic use-case for EctoDiff, let&#39;s look at a simple example. Assume you have two related ecto schemas like the following Pet with many Skills. Importantly, we&#39;ve chosen to cast_assoc the skills in the pet&#39;s changeset function, and we&#39;ve opted to use on_replace: :delete on the has_many skills association. defmodule Pet do use Ecto.Schema import Ecto.Changeset schema(&quot;pets&quot;) do field :name, :string field :type, :string, default: &quot;Cat&quot; has_many :skills, Skill, on_replace: :delete end def new(params), do: changeset(%__MODULE__{}, params) def update(struct, params), do: changeset(struct, params) defp changeset(struct, params) do struct |&gt; cast(params, [:name, :type]) |&gt; cast_assoc(:skills) end end defmodule Skill do use Ecto.Schema import Ecto.Changeset schema(&quot;skills&quot;) do field :name, :string field :level, :integer, default: 1 belongs_to :pet, Pet end def changeset(struct, params), do: cast(struct, params, [:name, :level]) end Now let&#39;s insert a pet into the database with three initial skills, defaulting to level: 1. {:ok, initial_pet} = %{name: &quot;Spot&quot;, skills: [%{name: &quot;Eating&quot;}, %{name: &quot;Sleeping&quot;}, %{name: &quot;Scratching&quot;}]} |&gt; Pet.new() |&gt; Repo.insert() Later, we&#39;ve decided to update this pet&#39;s name and it&#39;s skills. In this case, we&#39;re leaving &quot;eating&quot; alone (no changes), we&#39;re increasing &quot;sleeping&quot; to level: 2, we&#39;re implicitly deleting &quot;scratching&quot; by not including it in the list (taking advantage of on_replace: :delete), and we&#39;re adding a new skill &quot;meowing&quot;. [eating_id, sleeping_id, scratching_id] = Enum.map(initial_pet.skills, &amp; &amp;1.id) {:ok, updated_pet} = initial_pet |&gt; Pet.update(%{name: &quot;Spots&quot;, skills: [%{id: eating_id}, %{id: sleeping_id, level: 2}, %{name: &quot;Meowing&quot;}]}) |&gt; Repo.update() Now we can use EctoDiff to generate a data structure that describes all changes that occurred, making it easy to walk over all changes and act on them if desired. iex&gt; EctoDiff.diff(initial_pet, updated_pet) {:ok, #EctoDiff&lt; struct: EctoDiff.Pet, primary_key: %{id: 2}, effect: :changed, previous: #EctoDiff.Pet&lt;&gt;, current: #EctoDiff.Pet&lt;&gt;, changes: %{ name: {&quot;Spot&quot;, &quot;Spots&quot;}, skills: [ #EctoDiff&lt; struct: EctoDiff.Skill, primary_key: %{id: 5}, effect: :changed, previous: #EctoDiff.Skill&lt;&gt;, current: #EctoDiff.Skill&lt;&gt;, changes: %{level: {1, 2}} &gt;, #EctoDiff&lt; struct: EctoDiff.Skill, primary_key: %{id: 6}, effect: :deleted, previous: #EctoDiff.Skill&lt;&gt;, current: nil, changes: %{} &gt;, #EctoDiff&lt; struct: EctoDiff.Skill, primary_key: %{id: 7}, effect: :added, previous: #EctoDiff.Skill&lt;&gt;, current: #EctoDiff.Skill&lt;&gt;, changes: %{id: {nil, 7}, name: {nil, &quot;Meowing&quot;}, pet_id: {nil, 2}} &gt; ] } &gt;} Detailed documentation can be found at https://hexdocs.pm/ecto_diff."},{"ref":"changelog.html","title":"Changelog","module":"changelog","type":"extras","doc":"Changelog All notable changes to this project will be documented in this file. The format is based on Keep a Changelog and this project adheres to Semantic Versioning."},{"ref":"changelog.html#unreleased","title":"Changelog - Unreleased","module":"changelog","type":"extras","doc":"Nothing yet"},{"ref":"changelog.html#0-2-0-2019-04-06","title":"Changelog - 0.2.0 - 2019-04-06","module":"changelog","type":"extras","doc":"Added Documentation and typespecs to all public functions Basic documentation and usage to the Readme Credo to keep code healthy This changelog MIT License Code coverage tracking Dialyzer .travis.yml run all the above continuously"},{"ref":"changelog.html#0-1-0-2019-03-29","title":"Changelog - 0.1.0 - 2019-03-29","module":"changelog","type":"extras","doc":"Initial release"},{"ref":"license.html","title":"LICENSE","module":"license","type":"extras","doc":"The MIT License (MIT) Copyright 2019 Peek Travel, Inc. Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE."}]